//========================================================
//HI-MD Renderer Application - Windows Main function
//
//Written by Mark Collier.
//http://www.marcnetsystem.co.uk/
//
//This program is freeware and may be distributed in part or 
//in whole as long as a the author is credited clearly some way in 
//both the source code and in some visible way when the program 
//is running (eg:in a help screen or version info screen ).

//This notice must also appear in any source file that contains
//any part of the code in this file. 

//I (the author) take no responsability for the loss, transfer, corruption
//duplication, modification, or addition of legal or illegal data in any way. 
//
//Thanks for reading

//TO BUILD THIS YOU WILL NEED MICROSOFT DIRECTX SDK. 
//DOWNLOAD IT FROM THEM BEFORE EVEN ATTEMPTING TO BUILD THIS!

//MarC
//=========================================================

#include <windows.h>
#include <stdio.h>
#include "internal.h"

//=========================================================

#define HIMDRENDERCLASSNAME "HiMdRenderApplication"
#define HIMDRENDERTITLE		"MarC's HiMdRenderer - version 0.05a"
#define HIMDRENDERERERROR   "HiMdRenderer - ERROR"
#define HIMDRENDERERMSG		"HiMdRenderer"
#define RENDERBUTTON_OK		"GO RENDER!"
#define RENDERBUTTON_BUSY	"Please Wait..."

#define HIMDRENDER_GO		(WM_USER + 100)

//=========================Globals===========================
static char lpstrInFile[255];
static char lpstrOutFile[255];
static BOOL bProcessing = FALSE;

//window handles
static HWND hwndInputBrowse = NULL;
static HWND hwndOutputBrowse = NULL;
static HWND hwndInputFile = NULL;
static HWND hwndOutputFile = NULL;
static HWND hwndButton = NULL;


//========================Callbacks======================================
static void WindowMsgCallback(MARCWNDDATA mwnddata,HWND hwnd,int iMsg,WPARAM wparam,LPARAM lparam,LPVOID lpvParam)
{
	char lpstrTmpFile[255];
	char errbuffer[1024];
	int iRet;
	
	if(iMsg == HIMDRENDER_GO)
	{
		//init
		memset(errbuffer,0,sizeof(errbuffer));
		InitCore(lpstrInFile);

		//make temp filename
		strcpy(lpstrTmpFile,lpstrInFile);
		strcat(lpstrTmpFile,".tmp");
		
		//convert
		iRet = DoCoreConversion(lpstrInFile,lpstrTmpFile,lpstrOutFile,errbuffer);

		DeinitCore();
		if(iRet != 0 )
		{
			if(errbuffer[0] == 0)
			{
				sprintf(errbuffer,"An unknown error occured");
			}

			MessageBox(0,errbuffer,HIMDRENDERERERROR, MB_OK | MB_ICONEXCLAMATION |  MB_SETFOREGROUND);
		}
		else
		{
			MessageBox(0,"File converted!",HIMDRENDERERMSG, MB_OK | MB_ICONINFORMATION|  MB_SETFOREGROUND);
		}

		bProcessing = FALSE;
		MWNDSetText(hwndButton,RENDERBUTTON_OK);
	}
}

static void WindowEventCallback(MARCWNDDATA mwnddata,HWND hwnd,DWORD dwMsg,LPVOID lpvParam1,LPVOID lpvParam2)
{
	DWORD dwValue;
	char errbuffer[1024];

	switch((DWORD)lpvParam2)
	{
		case 0:
			if(hwnd == hwndInputBrowse)
			{
				if(GetInputFileFromAWindow(lpstrInFile,sizeof(lpstrInFile)))
				{
					strcpy(lpstrOutFile,lpstrInFile);
					strcat(lpstrOutFile,".wav");
					MWNDSetText(hwndInputFile,lpstrInFile);
					MWNDSetText(hwndOutputFile,lpstrOutFile);
				}
			}
			else if (hwnd == hwndOutputBrowse)
			{
				if(GetSaveFileFromAWindow(lpstrOutFile,sizeof(lpstrOutFile)))
				{
					MWNDSetText(hwndOutputFile,lpstrOutFile);
				}
			}
			else if((hwnd == hwndButton) && (!bProcessing))
			{	//get input/outfile files
				memset(lpstrInFile,0,sizeof(lpstrInFile));
				memset(lpstrOutFile,0,sizeof(lpstrOutFile));
				MWNDGetText(hwndInputFile,lpstrInFile,sizeof(lpstrInFile));
				MWNDGetText(hwndOutputFile,lpstrOutFile,sizeof(lpstrOutFile));

				//check settings
				if((lpstrInFile[0] == 0) || (!CheckFileIsOK(lpstrInFile,FALSE)))
				{
					MessageBox(0,"Invalid Input File!",HIMDRENDERERERROR, MB_OK | MB_ICONEXCLAMATION |  MB_SETFOREGROUND);
				}
				else if((lpstrOutFile[0] == 0) || (!CheckFileIsOK(lpstrOutFile,TRUE)))
				{
					MessageBox(0,"Invalid Destination File!",HIMDRENDERERERROR, MB_OK | MB_ICONEXCLAMATION |  MB_SETFOREGROUND);
				}
				else if(CheckSettings(errbuffer) != 0)
				{
					MessageBox(0,errbuffer,HIMDRENDERERERROR, MB_OK | MB_ICONEXCLAMATION |  MB_SETFOREGROUND);
				}
				else
				{	//all checks ok... lets do it,
					bProcessing = TRUE;
					MWNDSetText(hwndButton,RENDERBUTTON_BUSY);
					PostMessage(MWNDGetHWND(mwnddata),HIMDRENDER_GO,0,0);
				}

			}
			break;
		case 1:
			dwValue = GetSetting((DWORD)lpvParam1);
			SetSetting((DWORD)lpvParam1, !dwValue);
			MWNDSetCheckedState(hwnd,!dwValue);
			break;
		case 2:
			dwValue = MWNDGetTextNumber(hwnd);
			if ((DWORD)lpvParam1 == SETTING_SKIPSIZE) 
			{
				SetSetting((DWORD)lpvParam1, dwValue * 10000);
			}
			else
			{
				SetSetting((DWORD)lpvParam1, dwValue);
			}
			break;

	}
}

//=====================WinMain==========================================

int __stdcall WinMain(HINSTANCE hinst,HINSTANCE hprev,LPSTR cmd,int ishow)
{
	MARCWNDDATA mwndMainWindow;
	HWND hwnd;
	int y,i;
	struct _topts { DWORD opt;LPSTR desc; }tickopts[] = 
	{
		SETTING_KEEPTMPFILE , "Keep Temporary File", 
		SETTING_DEBUG,		  "Generate Log File",
		SETTING_USEAVGING,	  "Use average offsets",
		SETTING_NOIGNORELONGRPTS, "Cut average repeat length instead of ignoring large repeated blocks",
	};

	struct _nopts { DWORD opt;LPSTR desc;}numopts[] = 
	{
		SETTING_BLOCKSIZE	, "Block size (in seconds)",
		SETTING_SKIPSIZE	, "Overlap size (in milliseconds)",
		SETTING_ALLOWEDDIFF , "Maximum allowed byte difference",
	};

	mwndMainWindow = MWNDRegisterClass(hinst,HIMDRENDERCLASSNAME);
	if (mwndMainWindow == NULL)
	{
		return -1;
	}

	//create window
	if(!MWNDStartWindow(mwndMainWindow,600,350,HIMDRENDERTITLE))
	{
		return -2;
	}

	//add the controls for "input file"
	hwnd = MWNDAddControl(mwndMainWindow,CONTROLTYPE_STATIC,1,12,100,20,"Input File:",0,NULL,0,0);
	hwndInputFile =   MWNDAddControl(mwndMainWindow,CONTROLTYPE_EDITBOX,110,10,400,20,
									 "",CONTROLRESIZE_RESIZEWIDTH ,NULL,0,0);
	hwndInputBrowse = MWNDAddControl(mwndMainWindow,CONTROLTYPE_BUTTON,510,10,80,20,
									 "Browse...",CONTROLRESIZE_MOVEX,WindowEventCallback,0,0);
	MWNDSetControlFont(mwndMainWindow,hwndInputFile,16,8,FALSE,FALSE,"Courier new");
	MWNDSetControlFont(mwndMainWindow,hwndInputBrowse,16,8,FALSE,FALSE,"Courier new");
	MWNDSetControlFont(mwndMainWindow,hwnd,16,8,FALSE,FALSE,"Courier new");

	//add the controls for "output file"
	hwnd = MWNDAddControl(mwndMainWindow,CONTROLTYPE_STATIC,1,42,100,20,"Output File:",0,NULL,0,0);
	hwndOutputFile =   MWNDAddControl(mwndMainWindow,CONTROLTYPE_EDITBOX,110,40,400,20,
									 "",CONTROLRESIZE_RESIZEWIDTH ,NULL,0,0);
	hwndOutputBrowse = MWNDAddControl(mwndMainWindow,CONTROLTYPE_BUTTON,510,40,80,20,
									 "Browse...",CONTROLRESIZE_MOVEX,WindowEventCallback,0,0);
	MWNDSetControlFont(mwndMainWindow,hwndOutputFile,16,8,FALSE,FALSE,"Courier new");
	MWNDSetControlFont(mwndMainWindow,hwnd,16,8,FALSE,FALSE,"Courier new");
	MWNDSetControlFont(mwndMainWindow,hwndOutputBrowse,16,8,FALSE,FALSE,"Courier new");


	//add tick options
	y= 80;
	for(i=0; i < (sizeof(tickopts) / sizeof(_topts)) ; i++)
	{
		//add "keep tmp file" option
		hwnd = MWNDAddControl(mwndMainWindow,CONTROLTYPE_CHECKBOX,5,y,500,20,
							  tickopts[i].desc ,CONTROLRESIZE_RESIZEWIDTH,WindowEventCallback,
							  (LPVOID)tickopts[i].opt,(LPVOID)1);
		MWNDSetCheckedState(hwnd,GetSetting(tickopts[i].opt));
		MWNDSetControlFont(mwndMainWindow,hwnd,16,8,TRUE,FALSE,"Courier new");

		y += 25;
	}


	//add number options
	y+=30;
	for(i=0; i < (sizeof(numopts) / sizeof(_nopts)) ; i++)
	{
		//add "keep tmp file" option
		hwnd = MWNDAddControl(mwndMainWindow,CONTROLTYPE_STATIC,1,y,270,20,numopts[i].desc,CONTROLRESIZE_RESIZEWIDTH,NULL,0,0);
		MWNDSetControlFont(mwndMainWindow,hwnd,16,8,FALSE,FALSE,"Courier new");

		hwnd = MWNDAddControl(mwndMainWindow,CONTROLTYPE_EDITBOX,280,y,100,20,
							  tickopts[i].desc ,CONTROLRESIZE_MOVEX,WindowEventCallback,
							  (LPVOID)numopts[i].opt,(LPVOID)2);
		MWNDSetControlFont(mwndMainWindow,hwnd,16,8,FALSE,FALSE,"Courier new");

		if (numopts[i].opt == SETTING_SKIPSIZE) 
		{
			MWNDSetTextNumber(hwnd,GetSetting(numopts[i].opt) / 10000);
		}
		else
		{
			MWNDSetTextNumber(hwnd,GetSetting(numopts[i].opt));
		}
	
		y += 25;
	}


	hwndButton = MWNDAddControl(mwndMainWindow,CONTROLTYPE_BUTTON,200,290,200,25,RENDERBUTTON_OK,CONTROLRESIZE_MOVEX,WindowEventCallback,0,0);
	MWNDSetControlFont(mwndMainWindow,hwndButton,20,10,FALSE,FALSE,"Courier new");

	//set custom msg 
	MWNDSetCustomMsgCallback(mwndMainWindow,HIMDRENDER_GO,WindowMsgCallback,NULL);

	//and.... go!
	MWNDRunMsgLoop();


	MWNDUnregisterClass(mwndMainWindow);
	
	return 0;
}
