//========================================================
//HI-MD Renderer Application - window manipulation functions
//
//Written by Mark Collier.
//http://www.marcnetsystem.co.uk/
//
//This program is freeware and may be distributed in part or 
//in whole as long as a the author is credited clearly some way in 
//both the source code and in some visible way when the program 
//is running (eg:in a help screen or version info screen ).

//This notice must also appear in any source file that contains
//any part of the code in this file. 

//I (the author) take no responsability for the loss, transfer, corruption
//duplication, modification, or addition of legal or illegal data in any way. 
//
//Thanks for reading

//MarC
//=========================================================
#include <windows.h>
#include <stdio.h>
#include <commctrl.h>
#include "internal.h"

//=========================================================

#define MWNDM_CLICKEVENT WM_USER + 20

//*****************structures **************/

typedef struct _CONTROLENTRY
{
	HWND hwnd;
	DWORD dwControlType;
	LPCLICKCALLBACKPROC lpClickCallbackProc;
	LPVOID lpvParam1;
	LPVOID lpvParam2;
	HFONT hFont;

	float fX;
	float fY;
	float fWidth;
	float fHeight;

	DWORD dwResizeFlags;

}CONTROLENTRY,*LPCONTROLENTRY;


typedef struct _CUSTOMMSG
{
	int iMsg;
	LPVOID lpvParam;
	LPCUSTOMMSGCALLBACKPROC lpcmcbpCallbackProc;

}CUSTOMMSG, *LPCUSTOMMSG;

typedef struct _MAINWINDOWDATA
{
	//class name
	char acClassName[255];
	
	//instance
	HINSTANCE hInstance;
	
	//window handle
	HWND hwndWindowHandle;

	//window size 
	DWORD dwWidth;
	DWORD dwHeight;

	//controls
	struct _scontrols
	{
		HWND hwnd;
		CONTROLENTRY ctrlentry;
	}sControls[255];
	DWORD dwNumControls;

	//custom messages
	struct _smsgs
	{
		DWORD dwMsgValue;
		CUSTOMMSG custmsg;
	}sMsgs[255];
	DWORD dwNumMsgs;

} MAINWINDOWDATA, * LPMAINWINDOWDATA;

//-------------------------------Call-back functions--------------------------------
static LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	LPMAINWINDOWDATA lpmwndMainWndData;
	LPCREATESTRUCT lpcrsCreate;
	int  wmEvent;
	DWORD i,dwNewWidth,dwNewHeight,dwEvent;
	float fXScale,fYScale;
	HWND hwndCurrent;
	LPCONTROLENTRY lpce;
	LPCUSTOMMSG lpcmCustMsg;
	
	switch (message) 
	{
		case MWNDM_CLICKEVENT:
			dwEvent = (DWORD)lParam;
			wmEvent = 0xffff;
			hwndCurrent = (HWND)wParam;
		case WM_COMMAND:
			lpmwndMainWndData = (LPMAINWINDOWDATA)GetWindowLong(hWnd,GWL_USERDATA);
			
			if (message == WM_COMMAND)
			{
				wmEvent = HIWORD(wParam); 
				hwndCurrent = (HWND)lParam;//GetFocus();
				dwEvent = CLICKEVENT_LEFT;
			}
						
			//what command is this?
			switch(wmEvent)
			{
				case BN_DBLCLK:
					dwEvent = CLICKEVENT_DOUBLELEFT;
				case BN_CLICKED:
				case LBN_SELCHANGE:
				case EN_CHANGE:
				case 0xffff:
					//find window handle in table
					for(i=0;i<lpmwndMainWndData->dwNumControls;i++)
					{
						if(lpmwndMainWndData->sControls[i].hwnd == hwndCurrent)
						{
							lpce = &lpmwndMainWndData->sControls[i].ctrlentry;
							if( (lpce != NULL) && (lpce->lpClickCallbackProc != NULL) )
							{		//call callback
								lpce->lpClickCallbackProc(lpmwndMainWndData,hwndCurrent,dwEvent,lpce->lpvParam1,lpce->lpvParam2);
							}
						}
					}
					break;
				default:
					break;
			}
			break;
		case WM_INITDIALOG:
		case WM_CREATE: //window created... 
			//get and set main window structure info
			lpcrsCreate = (LPCREATESTRUCT)lParam;
			lpmwndMainWndData = (LPMAINWINDOWDATA)lpcrsCreate->lpCreateParams;
			SetWindowLong(hWnd,GWL_USERDATA,(DWORD)lpmwndMainWndData);
			
			break;

		case WM_DESTROY:
			lpmwndMainWndData = (LPMAINWINDOWDATA)GetWindowLong(hWnd,GWL_USERDATA);

			//quit
			PostQuitMessage(0);
			break;
		case WM_SIZE:
			lpmwndMainWndData = (LPMAINWINDOWDATA)GetWindowLong(hWnd,GWL_USERDATA);
			
			lpce = &lpmwndMainWndData->sControls[0].ctrlentry;
			
			//calculate new X and Y scale for controls
			dwNewWidth = (lParam & 0xffff) ? (lParam & 0xffff) : 1;
			dwNewHeight = (lParam >> 16) ? (lParam >> 16) : 1;
			fXScale =  (float)dwNewWidth /  (float)lpmwndMainWndData->dwWidth ;
			fYScale =  (float)dwNewHeight /  (float)lpmwndMainWndData->dwHeight ;  
			
			i=0;
			while(i < lpmwndMainWndData->dwNumControls )
			{
				//set controls new x,y,width,height
				if ((lpce->dwResizeFlags & CONTROLRESIZE_MOVEX) != 0)
				{
					lpce->fX *= fXScale;
				}

				if ((lpce->dwResizeFlags & CONTROLRESIZE_MOVEY) != 0)
				{
					lpce->fY *= fYScale;
				}

				if ((lpce->dwResizeFlags & CONTROLRESIZE_RESIZEHEIGHT) != 0)
				{
					if ((lpce->dwResizeFlags & CONTROLRESIZE_MOVEY) == 0)
					{
						lpce->fHeight = (lpce->fY + lpce->fHeight ) * fYScale;
						lpce->fHeight -= lpce->fY;

					}
					else
					{
						lpce->fHeight = lpce->fHeight * fYScale;
					}
				}

				if ((lpce->dwResizeFlags & CONTROLRESIZE_RESIZEWIDTH) != 0)
				{
					if ((lpce->dwResizeFlags & CONTROLRESIZE_MOVEX) == 0)
					{
						lpce->fWidth = (lpce->fX + lpce->fWidth) * fXScale;
						lpce->fWidth -= lpce->fX;

					}
					else
					{
						lpce->fWidth = lpce->fWidth * fXScale;
					}
				}
				
				SetWindowPos(lpce->hwnd,NULL,
							 (int)(lpce->fX ? lpce->fX : lpce->fX = 1),
							 (int)(lpce->fY ? lpce->fY : lpce->fY = 1),
							 (int)(lpce->fWidth ? lpce->fWidth : lpce->fWidth=1),
							 (int)(lpce->fHeight ? lpce->fHeight : lpce->fHeight=1),
							 SWP_NOZORDER | SWP_NOOWNERZORDER);
				
				lpce = &lpmwndMainWndData->sControls[++i].ctrlentry;
			}

			//set new window size
			lpmwndMainWndData->dwWidth = dwNewWidth;
			lpmwndMainWndData->dwHeight = dwNewHeight; 
			break;
		default:
			lpmwndMainWndData = (LPMAINWINDOWDATA)GetWindowLong(hWnd,GWL_USERDATA);
			lpcmCustMsg = NULL;

			if (lpmwndMainWndData != NULL)
			{
				for(i=0; i < lpmwndMainWndData->dwNumMsgs; i++)
				{
					if(lpmwndMainWndData->sMsgs[i].dwMsgValue == message)
					{
						lpcmCustMsg = &lpmwndMainWndData->sMsgs[i].custmsg;
					}
				}
			}

			if (  (lpcmCustMsg != NULL) && (lpcmCustMsg->lpcmcbpCallbackProc != NULL) )
			{
				lpcmCustMsg->lpcmcbpCallbackProc(lpmwndMainWndData,hWnd,message,wParam,lParam,lpcmCustMsg->lpvParam);
			}
			else
			{
				return DefWindowProc(hWnd, message, wParam, lParam);
			}
   }
   return 0;
}

//--------------------------------Exported Functions--------------------------------
MARCWNDDATA  MWNDRegisterClass(HINSTANCE hInstance,LPSTR lpstrClassName)
{
	LPMAINWINDOWDATA lpmwndMainWndData = (LPMAINWINDOWDATA)LocalAlloc(LPTR,sizeof(MAINWINDOWDATA));
	WNDCLASS wcex;
	ATOM atom;

	memset(lpmwndMainWndData,0,sizeof(MAINWINDOWDATA));

	strcpy(lpmwndMainWndData->acClassName,lpstrClassName);
	lpmwndMainWndData->hInstance = hInstance;

	//wcex.cbSize = sizeof(WNDCLASSEX); 
	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= (WNDPROC)WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= NULL;
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW);
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= lpmwndMainWndData->acClassName;
//	wcex.hIconSm		= NULL;

	atom = RegisterClass(&wcex);

	//init control tables
	lpmwndMainWndData->dwNumControls = 0;
	lpmwndMainWndData->dwNumMsgs = 0;
	

	//init common controls
	InitCommonControls();

	return lpmwndMainWndData;
}

void MWNDUnregisterClass(MARCWNDDATA mwnddata)
{
	LPMAINWINDOWDATA lpmwndMainWndData = (LPMAINWINDOWDATA)mwnddata;
	
	UnregisterClass(lpmwndMainWndData->acClassName,
				    lpmwndMainWndData->hInstance);

		
	LocalFree(lpmwndMainWndData);
}


BOOL MWNDStartWindow(MARCWNDDATA mwnddata,DWORD dwWidth,
					 DWORD dwHeight,LPSTR lpstrTitle)
{
	HWND hWnd;
	LPMAINWINDOWDATA lpmwndMainWndData = (LPMAINWINDOWDATA)mwnddata;

	lpmwndMainWndData->dwHeight = dwHeight;
	lpmwndMainWndData->dwWidth = dwWidth;

	//create main window
	hWnd = CreateWindow(lpmwndMainWndData->acClassName, 
						lpstrTitle, 
						WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN,
						CW_USEDEFAULT, 
						CW_USEDEFAULT, 
						dwWidth,
						dwHeight, 
						NULL, 
						NULL, 
						lpmwndMainWndData->hInstance, 
						(LPVOID)lpmwndMainWndData);

	//failed to create main window
	if (hWnd == 0)
	{
		return FALSE;
	}
	
	//set window handle in window data
	lpmwndMainWndData->hwndWindowHandle = hWnd;

	//show main window
	ShowWindow(hWnd, SW_SHOW);
	UpdateWindow(hWnd);

	return TRUE;
}

void  MWNDSetCustomMsgCallback(MARCWNDDATA wdWnd,int iMsg,
							  LPCUSTOMMSGCALLBACKPROC lpcmcbpCallbackProc,
							  LPVOID lpvParam)
{
	LPMAINWINDOWDATA lpmwndMainWndData = (LPMAINWINDOWDATA)wdWnd;
	CUSTOMMSG cmMsg;
	
	cmMsg.iMsg = iMsg;
	cmMsg.lpcmcbpCallbackProc = lpcmcbpCallbackProc;
	cmMsg.lpvParam = lpvParam;

	//add table entry
	memcpy(&lpmwndMainWndData->sMsgs[lpmwndMainWndData->dwNumMsgs].custmsg,
		   &cmMsg,sizeof(CUSTOMMSG));
	lpmwndMainWndData->sMsgs[lpmwndMainWndData->dwNumMsgs].dwMsgValue = (DWORD)iMsg;
	lpmwndMainWndData->dwNumMsgs++;

}

void  MWNDRunMsgLoop()
{
	MSG msg;

	//go into main window message loop
	while (GetMessage(&msg, NULL, 0, 0) > 0) 
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

}

HWND  MWNDGetHWND(MARCWNDDATA wdWndData)
{
	LPMAINWINDOWDATA lpmwndMainWndData = (LPMAINWINDOWDATA)wdWndData;
	return lpmwndMainWndData->hwndWindowHandle;
}


HWND  MWNDAddControl(MARCWNDDATA wdWndData,
					DWORD dwType,DWORD x,DWORD y,DWORD width,DWORD height,
					LPSTR lpszDisplayText,DWORD dwResizeFlags,
					LPCLICKCALLBACKPROC lpClickCallbackProc,LPVOID lpvPrm1,LPVOID lpvPrm2)
{
	HWND hwnd;
	CONTROLENTRY ceNew;
	LPMAINWINDOWDATA lpmwndMainWndData = (LPMAINWINDOWDATA)wdWndData;
	char szName[255];

	sprintf(szName,"CONTROL%d%d%d%d%d",dwType,x,y,width,height);

	switch (dwType)
	{
		case CONTROLTYPE_MULTILINEEDITBOX:
				hwnd = CreateWindow(TEXT("EDIT"), 
								"", 
								ES_MULTILINE | WS_CLIPSIBLINGS | WS_CHILD | WS_VISIBLE | WS_BORDER ,
								x,	
								y, 
								width,
								height, 
								lpmwndMainWndData->hwndWindowHandle, 
								NULL, 
								lpmwndMainWndData->hInstance,
								(LPVOID)lpmwndMainWndData);
			break;
		case CONTROLTYPE_STATIC:
			hwnd = CreateWindow(TEXT("STATIC"), 
								"", 
								WS_CLIPSIBLINGS | WS_CHILD | WS_VISIBLE  ,
								x,	
								y, 
								width,
								height, 
								lpmwndMainWndData->hwndWindowHandle, 
								NULL, 
								lpmwndMainWndData->hInstance,
								(LPVOID)lpmwndMainWndData);

			SetWindowText(hwnd,lpszDisplayText);
			break;

		case CONTROLTYPE_EDITBOX:
			hwnd = CreateWindow(TEXT("EDIT"), 
								"", 
								WS_CLIPSIBLINGS | WS_CHILD | WS_VISIBLE | WS_BORDER ,
								x,	
								y, 
								width,
								height, 
								lpmwndMainWndData->hwndWindowHandle, 
								NULL, 
								lpmwndMainWndData->hInstance,
								(LPVOID)lpmwndMainWndData);
			break;
		case CONTROLTYPE_LISTBOX:
			hwnd = CreateWindow(TEXT("LISTBOX"), 
								szName, 
								WS_CLIPSIBLINGS | WS_CHILD | WS_VISIBLE | WS_BORDER | LBS_NOTIFY ,
								x,	
								y, 
								width,
								height, 
								lpmwndMainWndData->hwndWindowHandle, 
								NULL, 
								lpmwndMainWndData->hInstance,
								(LPVOID)lpmwndMainWndData);
			
			ShowScrollBar(hwnd,SB_VERT,TRUE);
			break;
		case CONTROLTYPE_BUTTON:
			hwnd = CreateWindow(TEXT("BUTTON"), 
								szName, 
								WS_CLIPSIBLINGS | WS_CHILD | WS_VISIBLE | WS_BORDER ,
								x,	
								y, 
								width,
								height, 
								lpmwndMainWndData->hwndWindowHandle, 
								NULL, 
								lpmwndMainWndData->hInstance,
								(LPVOID)lpmwndMainWndData);

			SetWindowText(hwnd,lpszDisplayText);
			break;
		case CONTROLTYPE_CHECKBOX:
			hwnd = CreateWindow(TEXT("BUTTON"), 
								szName, 
								BS_CHECKBOX |  WS_CLIPSIBLINGS | WS_CHILD | WS_VISIBLE  ,
								x,	
								y, 
								width,
								height, 
								lpmwndMainWndData->hwndWindowHandle, 
								NULL, 
								lpmwndMainWndData->hInstance,
								(LPVOID)lpmwndMainWndData);

			SetWindowText(hwnd,lpszDisplayText);
			break;
	}
	
	ceNew.hwnd = hwnd;
	ceNew.lpClickCallbackProc = lpClickCallbackProc;
	ceNew.lpvParam1 = lpvPrm1;
	ceNew.lpvParam2 = lpvPrm2;
	ceNew.hFont = NULL;
	ceNew.fX = (float)x;
	ceNew.fY = (float)y;
	ceNew.fWidth = (float)width;
	ceNew.fHeight = (float)height;
	ceNew.dwResizeFlags = dwResizeFlags;
	ceNew.dwControlType = dwType;

	//add table entry
	memcpy(&lpmwndMainWndData->sControls[lpmwndMainWndData->dwNumControls].ctrlentry,
		   &ceNew,sizeof(CONTROLENTRY));
	lpmwndMainWndData->sControls[lpmwndMainWndData->dwNumControls].hwnd = hwnd;
	lpmwndMainWndData->dwNumControls++;
	
	return hwnd;
}

void  MWNDSetControlFont(MARCWNDDATA wdWnd,HWND hwndControl,int iFontHeight,int iFontWidth,
									  BOOL bItallic,BOOL bUnderline,LPSTR lpstrFontname)
{
	LPMAINWINDOWDATA lpmwndMainWndData = (LPMAINWINDOWDATA)wdWnd;
	LPCONTROLENTRY lpceCntEntry = NULL;
	DWORD i;

	for(i=0;i < lpmwndMainWndData->dwNumControls; i++)
	{
		if(lpmwndMainWndData->sControls[i].hwnd == hwndControl)
		{
			lpceCntEntry = &lpmwndMainWndData->sControls[i].ctrlentry;
		}
	}

	if (lpceCntEntry == NULL)
	{
		return;
	}

	//release current font - if
	if (lpceCntEntry->hFont != NULL)
	{
		DeleteObject(lpceCntEntry->hFont);
	}

	lpceCntEntry->hFont = CreateFont(iFontHeight,iFontWidth,0,0,FW_DONTCARE,
									 bItallic,bUnderline,FALSE,ANSI_CHARSET,
									 OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,
									 DEFAULT_QUALITY,FF_DONTCARE | FIXED_PITCH,
									 lpstrFontname);

	SendMessage(hwndControl,WM_SETFONT,(WPARAM)lpceCntEntry->hFont,(LPARAM)TRUE);
}

void  MWNDSetReadOnly(HWND hwnd,BOOL bReadOnly)
{
	SendMessage(hwnd,EM_SETREADONLY,bReadOnly,0);
}

void  MWNDSetText(HWND hwnd,LPSTR lpstrText)
{
	SendMessage(hwnd,WM_SETTEXT,0,(LPARAM)lpstrText);
}

void MWNDGetText(HWND hwnd,LPSTR lpstrBuf,DWORD bufsize)
{
	SendMessage(hwnd,WM_GETTEXT,bufsize,(LPARAM)lpstrBuf);
}

void MWNDSetTextNumber(HWND hwnd,DWORD dwValue)
{
	char text[255];

	sprintf(text,"%i",dwValue);
	SendMessage(hwnd,WM_SETTEXT,0,(LPARAM)text);
}

DWORD MWNDGetTextNumber(HWND hwnd)
{
	char text[255];

	SendMessage(hwnd,WM_GETTEXT,sizeof(text),(LPARAM)text);
	return strtoul(text,NULL,10);
}

DWORD  MWNDGetSelectedListParameter(HWND hwndListBox)
{
	int iIndex = SendMessage(hwndListBox,LB_GETCURSEL,0,0);

	if (iIndex == -1)
	{
		return 0xffffffff;
	}

	return SendMessage(hwndListBox,LB_GETITEMDATA,iIndex,0);
}

void  MWNDGetSelectedListText(HWND hwndListBox,LPSTR lpstrBuffer)
{
	int iIndex = SendMessage(hwndListBox,LB_GETCURSEL,0,0);

	if (iIndex == -1)
	{
		return;
	}

	SendMessage(hwndListBox,LB_GETTEXT,iIndex,(LPARAM)lpstrBuffer);
}

void  MWNDSetSelectedListIndex(HWND hwndListBox,int iIndex,BOOL bCallClickCallback)
{
	if (iIndex == -1)
	{
		iIndex = SendMessage(hwndListBox,LB_GETCOUNT,0,0) - 1;
	}

	SendMessage(hwndListBox,LB_SETCURSEL,iIndex,0);

	if (bCallClickCallback)
	{
		SendMessage(GetParent(hwndListBox),WM_COMMAND,LBN_SELCHANGE << 16,(LPARAM)hwndListBox);
	}
}


void  MWNDClearList(HWND hwndListBox)
{
	SendMessage(hwndListBox,LB_RESETCONTENT,0,0);
}

void  MWNDAddToList(HWND hwndListBox,LPSTR lpstrString,DWORD dwData)
{
	int iIndex;

	iIndex = SendMessage(hwndListBox,LB_ADDSTRING,0,(LPARAM)lpstrString);
	SendMessage(hwndListBox,LB_SETITEMDATA,iIndex,dwData);
}


void MWNDSetCheckedState(HWND hwndCheckBox,BOOL bChecked)
{
	SendMessage(hwndCheckBox,BM_SETCHECK, bChecked ? BST_CHECKED : BST_UNCHECKED,0);
}