//========================================================
//HI-MD Renderer Application - Command-line MAIN function
//
//Written by Mark Collier.
//http://www.marcnetsystem.co.uk/
//
//This program is freeware and may be distributed in part or 
//in whole as long as a the author is credited clearly some way in 
//both the source code and in some visible way when the program 
//is running (eg:in a help screen or version info screen ).

//This notice must also appear in any source file that contains
//any part of the code in this file. 

//I (the author) take no responsability for the loss, transfer, corruption
//duplication, modification, or addition of legal or illegal data in any way. 
//
//Thanks for reading

//MarC
//=================================================================

#include <windows.h>
#include <stdio.h>
#include <conio.h>
#include "internal.h"

//===========================Globals================================
//These are settings only used by the command-line version.
//So there is no need to put them into Settings.cpp
static char infile[MAX_PATH];
static char tempfile[MAX_PATH];
static char destfile[MAX_PATH];
static DWORD bWaitKey = TRUE;

//========================Utility functions===========================
static LPSTR FindFirstNonSpace(char * x)
{
	
	if(x == NULL)
	{
		return 0;
	}
	
	while((x[0] == ' ') && (x[0] != 0))
	{
		x++;
	}

	return x;
}

static BOOL ProcessArg(char * x,char * y)
{
	char * param;
	DWORD dwValue;

	if((x[0] == '-'))
	{	
		switch(x[1])
		{
			case 'i':	
				param = FindFirstNonSpace(x+2);
				if((param == NULL) || (param[0] == 0))
				{
					param = FindFirstNonSpace(y);	
				}

				if(param != 0)
				{
					strcpy(infile,param);
				}

				if(!CheckFileIsOK(infile,FALSE))
				{
					printf("ERROR: Could not open input file %s \n",infile);
					return FALSE;
				}
				break;	
			case 'o':		
				param = FindFirstNonSpace(x+2);
				if((param == NULL) || (param[0] == 0))
				{
					param = FindFirstNonSpace(y);	
				}

				if(param != 0)
				{
					strcpy(destfile,param);
				}

				if(!CheckFileIsOK(destfile,TRUE))
				{
					printf("ERROR: Could not open destination file %s \n",destfile);
					return FALSE;
				}
				break;
			case 't':
				param = FindFirstNonSpace(x+2);
				if((param == NULL) || (param[0] == 0))
				{
					param = FindFirstNonSpace(y);	
				}

				if(param != 0)
				{
					strcpy(tempfile,param);
				}

				if(!CheckFileIsOK(tempfile,TRUE))
				{
					printf("ERROR: Could not open temp file %s \n",tempfile);
					return FALSE;
				}
				break;
			case 'k':
				SetSetting(SETTING_KEEPTMPFILE,TRUE);
				break;
			case 'd':
				SetSetting(SETTING_DEBUG, TRUE);
				break;
			case 'c':
				SetSetting(SETTING_USEAVGING,TRUE);
				break;
			case 'f':
				param = FindFirstNonSpace(x+2);
				if((param == NULL) || (param[0] == 0))
				{
					param = FindFirstNonSpace(y);	
				}

				if(param == NULL)
				{
					printf("ERROR: Invalid specified maximum difference value ");
					return FALSE;
				}

				SetSetting(SETTING_ALLOWEDDIFF,strtol(param,NULL,10));
				break;
			case 'b':
				param = FindFirstNonSpace(x+2);
				if((param == NULL) || (param[0] == 0))
				{
					param = FindFirstNonSpace(y);	
				}
				
				dwValue = strtol(param,NULL,10);

				SetSetting(SETTING_BLOCKSIZE,dwValue);
				break;
			case 's':
				param = FindFirstNonSpace(x+2);
				if((param == NULL) || (param[0] == 0))
				{
					param = FindFirstNonSpace(y);	
				}
				
				dwValue = strtol(param,NULL,10);

				SetSetting(SETTING_SKIPSIZE,dwValue * 10000);
				break;
			case 'e':
				bWaitKey = FALSE;
				break;
			case 'l':
				SetSetting(SETTING_NOIGNORELONGRPTS,FALSE);
				break;
			default:
				printf("Invalid parameter!\n");
			case 'h':
				printf("===============================================================================\n");
				printf("|               MarC's slightly lese dodgy OMA -> WAV converter ... v0.05     |\n");
				printf("===============================================================================\n");
				printf("  -h              Displays this help\n");
				printf("  -i <file>       Specifies input file \n");
				printf("  -o <file>       Specifies output file [OPTIONAL] \n");
				printf("  -t <file>       Specifies temporary file [OPTIONAL] \n");
				printf("  -k              Keep temporary file [OPTIONAL] \n");
				printf("  -b <block size> Read block size in seconds [OPTIONAL] default=%u\n",GetSetting(SETTING_BLOCKSIZE));
				printf("  -f <max value>  Max allowed difference between samples [OPTIONAL] default=%u\n",GetSetting(SETTING_ALLOWEDDIFF));
				printf("  -s <skip value> No.of milliseconds to overlap blocks [OPTIONAL] default=%u\n",GetSetting(SETTING_SKIPSIZE) / 10000);
				printf("  -d              Produce a log file full of debug [OPTIONAL]\n");
				printf("  -c              Use old non-averaging method of offset calculation [OPTIONAL]\n");
				printf("  -e              Dont wait for keypress after conversion [OPTIONAL].\n");
				printf("  -l              Ignore larger than allowed sizes of repeated audio [OPTIONAL]\n");
				printf("===============================================================================\n");
				return FALSE;
		}
	}

	return TRUE;
}

//========================main main main main===========================

int main(int argv,char ** argc)
{
	int iRet = -1;
	int i;
	char errbuffer[1024];

	//init
	memset(errbuffer,0,sizeof(errbuffer));
	memset(infile,0,sizeof(infile));
	memset(tempfile,0,sizeof(tempfile));
	memset(destfile,0,sizeof(destfile));

	//process command params
	for(i=1;i < argv;i++)
	{
		if(i != (argv-1))
		{
			if(!ProcessArg(argc[i],argc[i+1]))
			{
				return 0;
			}
		}
		else
		{
			if(!ProcessArg(argc[i],NULL))
			{
				return 0;
			}
		}
	}

	//check and get input file
	if(infile[0] == 0)
	{
		if((!GetInputFileFromAWindow(infile,sizeof(infile))) || (infile[0] ==0))
		{
			printf("ERROR: No input file!\n");
			ProcessArg("-h",NULL);
			if(bWaitKey)
			{
				printf("\n\nPress any key to exit");
				getch();
			}
			return -1;
		}

		//force log output in this mode
		SetSetting(SETTING_DEBUG,TRUE);
	}
	
	//set up temp file name
	if(tempfile[0] == 0)
	{
		//apend .wav.tmp for temporary file
		sprintf(tempfile,"%s.wav.tmp",infile);
	}

	//set up dest file name
	if(destfile[0] == 0)
	{
		//append .wav for output wav
		sprintf(destfile,"%s.wav",infile);
	}

	//check settings
	if(CheckSettings(errbuffer) != 0)
	{
		printf(errbuffer);
		printf("\n");
		return -2;
	}

	//init the core and give it the name of the log file to use
	InitCore(infile);

	iRet = DoCoreConversion(infile,tempfile,destfile,errbuffer);
	if(iRet != 0)
	{
		printf("\n\n==================================================================\n");
		printf(errbuffer);
		printf("\n");
	}
	
	DeinitCore();

	if(bWaitKey)
	{
		printf("\n\nResult:%u Press anything to exit",iRet);
		getch();
	}

	return 0;

}
